//***************************************************************************
//TCrypZLib v1.00
//  TCyptZLib is derived from TCryptFile by Giovanni Burzomato(burzox@tin.it).
//  I have changed the original routine slightly but the encryption remains
//  basically as it is in TCryptFile.  I've added ZLib stream compression
//  with the option to choose the compression level that suits you.
//  This is more of a component writing exercise than a serious attempt to
//  create a secure component, though a combination of compression and simple
//  encryption does seem to create an interestingly garbled file!! It
//  would be interesting to know what the comparitive strength of it is?
//  This code is FREEWARE and you can do with it what you will.
//  If you have any suggestions to improve or enhance the code, please let me know.
//
//  FileName: CryptZLib.pas (TCryptZLib v1.0)
//  Author: Brian Shepherd
//  Date: 30/07/99
//  email: bshepherd@mail.com
//***************************************************************************
//
//TCryptFile : Delphi 16 - 32 (Taken from original file)

//Is a simple and very fast Encrypt/Decrypt component working
//on registers with a 64 bits key.
//It looks like very much TFileCopy, (another component by me for
//files copy) since it manages binary files the same way.
//Just on buffering data it performs encrypting &/or decrypting.
//Because of binary management it allows encoding on any kind of
//files.
//The encoding algorithm is very easy and you can understand it just
//reading sources, but there are two keys to use to build an
//application, one for the developer and one for the end-user.
//So, also having sources, it will be difficult (sure, not
//impossible ... but that's normal with warez people ...) to
//crack the machine.
//FIntKey is the key that developers can set at design time by
//object inspector so any application can be different in order
//of a 32 bit range.
//FKey is the 4 chars user's password encoded by FIntKey so,
//finally you have a 64 bits encoding.
//FKey is put at the beginning of the encoded file so it encreases
//it's length by 4 bytes and the decrypt method performs a check
//with user's password and the one stored on data.


UNIT CryptZLib;

INTERFACE

USES
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ZLib;

CONST
  BlockSize = $4000;
  BuffSize = 4096;

TYPE

  pBuf = ^tBuf;
  tBuf = ARRAY[1..BlockSize] OF byte;
  TCompression = (zNone, zFast, zMax);
  EInputError = CLASS(Exception);

  TCryptZlib = CLASS(TComponent)
  private
    { Private declarations }

    fInFile: STRING;
    fOutFile: STRING;
    fKey: LongInt; { seed }
    fComp: TCompression;
    fCompress: TCompressionLevel;
    fIntKey: LongInt; { internal key    }
    fPassword: STRING; { string password }
    buf: pBuf;
    nRead: integer;
    nWritten: integer;
    Buffer: ARRAY[0..BuffSize - 1] OF Byte;
    Source, Target, Temp: TStream;


    PROCEDURE SetCompress(value: TCompression);
    PROCEDURE WriteFPassword(value: STRING);
    PROCEDURE WriteFIntKey(value: LongInt);
    PROCEDURE InitKey;
    FUNCTION CheckTarget: boolean;
  protected

    PROCEDURE CompressStream(Stream, SaveStream: TStream);
    PROCEDURE DeCompressStream(Stream, CompressedStream: TStream);
    { Protected declarations }
  public
    { Public declarations }

    CONSTRUCTOR Create(AOwner: TComponent); override;
    PROCEDURE Encrypt;
    PROCEDURE Decrypt;

  published
    { Published declarations }
    PROPERTY Infile: STRING read FInFile write FInFile;
    PROPERTY Outfile: STRING read FOutFile write FOutFile;
    PROPERTY Internal_Key: LongInt read fIntKey write WriteFIntKey;
    PROPERTY Password: STRING read FPassword write WriteFPassword;
    PROPERTY Compression: TCompression read fComp write SetCompress DEFAULT zFast;


  END;

PROCEDURE Register;

IMPLEMENTATION

PROCEDURE Register;
BEGIN
  RegisterComponents('Additional', [TCryptZlib]);
END;

CONSTRUCTOR TCryptZlib.Create(AOwner: TComponent);
BEGIN
  INHERITED Create(AOwner);
  Password := '';
  fComp := zFast;
  fIntKey := 170244;
END;

PROCEDURE TCryptZlib.SetCompress(value: TCompression);
BEGIN
  IF fComp <> value THEN
    fComp := value;
  CASE value OF
    zNone: fCompress := clNone;
    zFast: fCompress := clFastest;
    zMax: fCompress := clMax;
  END;
END;

PROCEDURE TCryptZlib.CompressStream(Stream, SaveStream: TStream);
VAR
  CompStrm: TCompressionStream;
BEGIN
  CompStrm := TCompressionStream.Create(fCompress, SaveStream);
  TRY
    Stream.Seek(0, 0);
    CompStrm.CopyFrom(Stream, 0);
  FINALLY
    CompStrm.Free;
  END;

END;

PROCEDURE TCryptZlib.DeCompressStream(Stream, CompressedStream: TStream);
VAR
  size: LongInt;
  DecompStrm: TDecompressionStream;
BEGIN
  CompressedStream.Seek(0, 0);
  DeCompStrm := TDecompressionStream.Create(CompressedStream);
  TRY
    WHILE True DO BEGIN
      Size := DecompStrm.Read(Buffer, BuffSize);
      IF Size <> 0 THEN Stream.WriteBuffer(Buffer, Size) ELSE Break;
    END;
  FINALLY
    DecompStrm.Free;

  END;
END;

PROCEDURE TCryptZlib.WriteFPassword(value: STRING);
BEGIN
  IF length(value) > 4 THEN
  BEGIN
    RAISE EInputError.Create('Max. length 4 characters...');
    exit;
  END;
  FPassword := value;
END;

PROCEDURE TCryptZlib.WriteFIntKey(value: LongInt);
BEGIN
  IF (value = 0) OR (value = $FFFFFFFF) THEN
  BEGIN
    RAISE EInputError.Create('Value not allowed');
    exit;
  END;
  FIntKey := value;
END;


PROCEDURE TCryptZlib.InitKey;
VAR
  cbuf: ARRAY[0..4] OF char;
BEGIN
  IF FPassword = '' THEN FKey := 0
  ELSE
  BEGIN
    fillChar(cbuf, SizeOf(cbuf), #0);
    StrPCopy(cbuf, FPassword);
    move(cbuf, FKey, sizeOf(FKey));
  END;

  RandSeed := FKey;
END;

FUNCTION TCryptZlib.CheckTarget: boolean;
BEGIN
  Result := False;

  IF FInFile = FOutFile THEN
  BEGIN
    RAISE EInputError.Create('You cannot use the source file as the target file..');
    Exit;
  END;

  IF fileExists(FOutFile) THEN
  BEGIN
    IF MessageDlg('The target file : ' + ExtractFileName(FOutFile) + ' already exists.' + chr(13) +
      'Any data will be lost.' + ' PROCEED ?'
      , mtWarning, [mbYes, mbNo], 0) = mrNo THEN
      exit
    ELSE
      IF NOT DeleteFile(fOutFile) THEN
      BEGIN
        RAISE EInputError.Create('Unable to delete the target file..');
        exit;
      END;
  END;

  TRY
    Target := TFileStream.Create(FOutFile, fmCreate);
  EXCEPT
    ON E: Exception DO
    BEGIN
      ShowMessage(E.Message);
      Target.Free;
      Exit;
    END;
  END;
  Result := TRUE;

END;

PROCEDURE TCryptZlib.Encrypt;
VAR
  cont: word;
BEGIN

  IF CheckTarget THEN
  TRY
    TRY
      Source := TFileStream.Create(fInFile, fmOpenRead OR fmShareExclusive);
      Temp := TMemoryStream.Create;
      CompressStream(Source, Temp);
    FINALLY
      Source.Free;

    END;
    new(buf);

    InitKey;

    Temp.Seek(0, 0);
    Target.Seek(0, 0);

    inc(FKey, FIntKey);

    nWritten := Target.Write(FKey, SizeOf(FKey));

    REPEAT
      BEGIN
        nRead := Temp.Read(Buf^, sizeOf(buf^));

        FOR cont := 1 TO nRead DO inc(buf^[cont], random(cont));
        nWritten := Target.Write(Buf^, nRead);
      END;
    UNTIL (nRead = 0) OR (nWritten <> nRead);

  FINALLY
    Dispose(buf);
    Target.Free;
    Temp.Free;
  END;
END;

PROCEDURE TCryptZlib.Decrypt;
VAR
  cont: word;
  FileKey: longInt;
BEGIN

  Source := TFileStream.Create(FInfile, fmOpenRead OR fmShareExclusive);
  Source.Seek(0, 0);

  InitKey;

  nRead := Source.Read(FileKey, sizeOf(FKey));

  dec(FileKey, FIntKey);

  IF FileKey <> FKey THEN
  BEGIN
    Source.Free;
    Target.Free;
    RAISE EInputError.Create('Bad password or file is not encrypted!');
    exit;
  END;

  IF NOT CheckTarget THEN
    Source.Free
  ELSE

  TRY

    Temp := TMemoryStream.Create;

    new(buf);

    REPEAT
      BEGIN
        nRead := Source.Read(Buf^, sizeOf(buf^));

        FOR cont := 1 TO nRead DO dec(buf^[cont], random(cont));
        nWritten := Temp.Write(Buf^, nRead);
      END;
    UNTIL (nRead = 0) OR (nWritten <> nRead);

    Temp.Seek(0, 0);
    Target.Seek(0, 0);

    DeCompressStream(Target, Temp);

  FINALLY
    Temp.Free;
    Source.Free;
    Target.Free;
    Dispose(buf);
  END;

END;

END.

